Stan Math Library: stan/math/prim/scal/prob/exp_mod_normal_cdf.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

exp_mod_normal_cdf.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_EXP_MOD_NORMAL_CDF_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_EXP_MOD_NORMAL_CDF_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/meta/include_summand.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <boost/random/normal_distribution.hpp>
15 #include <boost/math/special_functions/fpclassify.hpp>
16 #include <boost/random/variate_generator.hpp>
17 #include <cmath>
18
19 namespace stan {
20
21 namespace math {
22
23 template <typename T_y, typename T_loc, typename T_scale,
24 typename T_inv_scale>
25 typename return_type<T_y, T_loc, T_scale, T_inv_scale>::type
26 exp_mod_normal_cdf(const T_y& y, const T_loc& mu, const T_scale& sigma,
27 const T_inv_scale& lambda) {
28 static const char* function("stan::math::exp_mod_normal_cdf");
29 typedef typename stan::partials_return_type<T_y, T_loc, T_scale,
30 T_inv_scale>::type
31 T_partials_return;
32
33 using stan::math::check_positive_finite;
34 using stan::math::check_finite;
35 using stan::math::check_not_nan;
36 using stan::math::check_consistent_sizes;
37 using stan::math::value_of;
38
39 T_partials_return cdf(1.0);
40 // check if any vectors are zero length
41 if (!(stan::length(y)
42 && stan::length(mu)
43 && stan::length(sigma)
44 && stan::length(lambda)))
45 return cdf;
46
47 check_not_nan(function, "Random variable", y);
48 check_finite(function, "Location parameter", mu);
49 check_not_nan(function, "Scale parameter", sigma);
50 check_positive_finite(function, "Scale parameter", sigma);
51 check_positive_finite(function, "Inv_scale parameter", lambda);
52 check_not_nan(function, "Inv_scale parameter", lambda);
53 check_consistent_sizes(function,
54 "Random variable", y,
55 "Location parameter", mu,
56 "Scale parameter", sigma,
57 "Inv_scale paramter", lambda);
58
59 OperandsAndPartials<T_y, T_loc, T_scale, T_inv_scale>
60 operands_and_partials(y, mu, sigma, lambda);
61
62 using stan::math::SQRT_2;
63 using std::exp;
64
65 VectorView<const T_y> y_vec(y);
66 VectorView<const T_loc> mu_vec(mu);
67 VectorView<const T_scale> sigma_vec(sigma);
68 VectorView<const T_inv_scale> lambda_vec(lambda);
69 size_t N = max_size(y, mu, sigma, lambda);
70 const double sqrt_pi = std::sqrt(stan::math::pi());
71 for (size_t n = 0; n < N; n++) {
72 if (boost::math::isinf(y_vec[n])) {
73 if (y_vec[n] < 0.0)
74 return operands_and_partials.value(0.0);
75 }
76
77 const T_partials_return y_dbl = value_of(y_vec[n]);
78 const T_partials_return mu_dbl = value_of(mu_vec[n]);
79 const T_partials_return sigma_dbl = value_of(sigma_vec[n]);
80 const T_partials_return lambda_dbl = value_of(lambda_vec[n]);
81 const T_partials_return u = lambda_dbl * (y_dbl - mu_dbl);
82 const T_partials_return v = lambda_dbl * sigma_dbl;
83 const T_partials_return v_sq = v * v;
84 const T_partials_return scaled_diff = (y_dbl - mu_dbl) / (SQRT_2
85 * sigma_dbl);
86 const T_partials_return scaled_diff_sq = scaled_diff * scaled_diff;
87 const T_partials_return erf_calc = 0.5 * (1 + erf(-v / SQRT_2
88 + scaled_diff));
89 const T_partials_return deriv_1 = lambda_dbl * exp(0.5 * v_sq - u)
90 * erf_calc;
91 const T_partials_return deriv_2 = SQRT_2 / sqrt_pi * 0.5
92 * exp(0.5 * v_sq - (scaled_diff - (v / SQRT_2))
93 * (scaled_diff - (v / SQRT_2)) - u) / sigma_dbl;
94 const T_partials_return deriv_3 = SQRT_2 / sqrt_pi * 0.5
95 * exp(-scaled_diff_sq) / sigma_dbl;
96
97 const T_partials_return cdf_ = 0.5 * (1 + erf(u / (v * SQRT_2)))
98 - exp(-u + v_sq * 0.5) * (erf_calc);
99
100 cdf *= cdf_;
101
102 if (!is_constant_struct<T_y>::value)
103 operands_and_partials.d_x1[n] += (deriv_1 - deriv_2 + deriv_3)
104 / cdf_;
105 if (!is_constant_struct<T_loc>::value)
106 operands_and_partials.d_x2[n] += (-deriv_1 + deriv_2 - deriv_3)
107 / cdf_;
108 if (!is_constant_struct<T_scale>::value)
109 operands_and_partials.d_x3[n] += (-deriv_1 * v - deriv_3
110 * scaled_diff * SQRT_2 - deriv_2
111 * sigma_dbl * SQRT_2
112 * (-SQRT_2 * 0.5
113 * (-lambda_dbl + scaled_diff
114 * SQRT_2 / sigma_dbl) - SQRT_2
115 * lambda_dbl)) / cdf_;
116 if (!is_constant_struct<T_inv_scale>::value)
117 operands_and_partials.d_x4[n] += exp(0.5 * v_sq - u)
118 * (SQRT_2 / sqrt_pi * 0.5 * sigma_dbl
119 * exp(-(v / SQRT_2 - scaled_diff) * (v / SQRT_2 - scaled_diff))
120 - (v * sigma_dbl + mu_dbl - y_dbl) * erf_calc) / cdf_;
121 }
122
123 if (!is_constant_struct<T_y>::value) {
124 for (size_t n = 0; n < stan::length(y); ++n)
125 operands_and_partials.d_x1[n] *= cdf;
126 }
127 if (!is_constant_struct<T_loc>::value) {
128 for (size_t n = 0; n < stan::length(mu); ++n)
129 operands_and_partials.d_x2[n] *= cdf;
130 }
131 if (!is_constant_struct<T_scale>::value) {
132 for (size_t n = 0; n < stan::length(sigma); ++n)
133 operands_and_partials.d_x3[n] *= cdf;
134 }
135 if (!is_constant_struct<T_inv_scale>::value) {
136 for (size_t n = 0; n < stan::length(lambda); ++n)
137 operands_and_partials.d_x4[n] *= cdf;
138 }
139
140 return operands_and_partials.value(cdf);
141 }
142 }
143 }
144 #endif
145
146
147
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::sqrt
fvar< T > sqrt(const fvar< T > &x)
Definition: sqrt.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
stan::math::exp_mod_normal_cdf
return_type< T_y, T_loc, T_scale, T_inv_scale >::type exp_mod_normal_cdf(const T_y &y, const T_loc &mu, const T_scale &sigma, const T_inv_scale &lambda)
Definition: exp_mod_normal_cdf.hpp:26
stan::math::erf
fvar< T > erf(const fvar< T > &x)
Definition: erf.hpp:14
is_constant_struct.hpp
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::SQRT_2
const double SQRT_2
The value of the square root of 2, .
Definition: constants.hpp:21
boost::math::isinf
bool isinf(const stan::math::var &v)
Checks if the given number is infinite.
Definition: boost_isinf.hpp:22
stan::partials_return_type
Definition: partials_return_type.hpp:16
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::pi
double pi()
Return the value of pi.
Definition: constants.hpp:86
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::OperandsAndPartials::d_x4
VectorView< T_return_type, false, true > d_x4
Definition: OperandsAndPartials.hpp:42

[ Stan Home Page ] © 2011–2016, Stan Development Team.